import os
from typing import Any, Dict

import supervisely.io.env as sly_env
import supervisely.nn.training.gui.utils as gui_utils
from supervisely import Api, logger
from supervisely._utils import is_production
from supervisely.api.api import ApiField
from supervisely.app.widgets import (
    Card,
    Container,
    Empty,
    Field,
    Flexbox,
    FolderThumbnail,
    ReportThumbnail,
    Text,
)
from supervisely.io.fs import file_exists

PYTORCH_ICON = "https://img.icons8.com/?size=100&id=jH4BpkMnRrU5&format=png&color=000000"
ONNX_ICON = "https://artwork.lfaidata.foundation/projects/onnx/icon/color/onnx-icon-color.png"
TRT_ICON = "https://img.icons8.com/?size=100&id=yqf95864UzeQ&format=png&color=000000"

OVERVIEW_FILE_NAME = "README.md"
PYTORCH_FILE_NAME = "demo_pytorch.py"
ONNX_FILE_NAME = "demo_onnx.py"
TRT_FILE_NAME = "demo_tensorrt.py"


class TrainingArtifacts:
    title = "Training Artifacts"
    description = "All outputs of the training process will appear here"
    lock_message = "Artifacts will be available after training is completed"

    def __init__(self, api: Api, app_options: Dict[str, Any]):
        self.display_widgets = []
        self.success_message_text = (
            "Training completed. Training artifacts were uploaded to Team Files. "
            "You can find and open tensorboard logs in the artifacts folder via the "
            "<a href='https://ecosystem.supervisely.com/apps/tensorboard-experiments-viewer' target='_blank'>Tensorboard Experiment Viewer</a> app."
        )
        self.app_options = app_options

        # GUI Components
        self.validator_text = Text("")
        self.validator_text.hide()
        self.display_widgets.extend([self.validator_text])

        # Outputs
        self.artifacts_thumbnail = FolderThumbnail()
        self.artifacts_thumbnail.hide()

        self.artifacts_field = Field(
            title="Artifacts",
            description="Contains all outputs of the training process",
            content=self.artifacts_thumbnail,
        )
        self.artifacts_field.hide()
        self.display_widgets.extend([self.artifacts_field])

        # Optional Model Benchmark
        if app_options.get("model_benchmark", False):
            self.model_benchmark_report_thumbnail = ReportThumbnail()
            self.model_benchmark_report_thumbnail.hide()

            self.model_benchmark_fail_text = Text(
                text="Model evaluation did not finish successfully. Please check the app logs for details.",
                status="error",
            )
            self.model_benchmark_fail_text.hide()

            self.model_benchmark_widgets = Container(
                [self.model_benchmark_report_thumbnail, self.model_benchmark_fail_text]
            )

            self.model_benchmark_report_field = Field(
                title="Model Benchmark",
                description="Evaluation report of the trained model",
                content=self.model_benchmark_widgets,
            )
            self.model_benchmark_report_field.hide()
            self.display_widgets.extend([self.model_benchmark_report_field])
        # -------------------------------- #

        # PyTorch, ONNX, TensorRT demo
        self.inference_demo_widgets = []

        model_demo = self.app_options.get("demo", None)
        if model_demo is not None:
            model_demo_path = model_demo.get("path", None)
            if model_demo_path is not None:
                model_demo_gh_link = None
                self.pytorch_instruction = None
                self.onnx_instruction = None
                self.trt_instruction = None
                if is_production():
                    task_id = sly_env.task_id()
                    task_info = api.task.get_info_by_id(task_id)
                    app_id = task_info["meta"]["app"]["id"]
                    app_info = api.app.get_info_by_id(app_id)
                    model_demo_gh_link = app_info.repo
                else:
                    app_name = sly_env.app_name()
                    module_info = gui_utils.get_module_info_by_name(api, app_name)
                    if module_info is not None:
                        model_demo_gh_link = module_info["repo"]
                    else:
                        logger.warning(
                            f"App '{app_name}' not found in Supervisely Ecosystem. Demo artifacts will not be displayed."
                        )

                if model_demo_gh_link is not None:
                    gh_branch = "blob/main"
                    link_to_demo = f"{model_demo_gh_link}/{gh_branch}/{model_demo_path}"

                    if model_demo_gh_link is not None and model_demo_path is not None:
                        # PyTorch
                        local_pytorch_demo = os.path.join(
                            os.getcwd(), model_demo_path, PYTORCH_FILE_NAME
                        )
                        if file_exists(local_pytorch_demo):
                            pytorch_demo_link = f"{link_to_demo}/{PYTORCH_FILE_NAME}"
                            pytorch_icon = Field.Icon(
                                image_url=PYTORCH_ICON, bg_color_rgb=[255, 255, 255]
                            )
                            self.pytorch_instruction = Field(
                                title="PyTorch",
                                description="Open file",
                                description_url=pytorch_demo_link,
                                icon=pytorch_icon,
                                content=Empty(),
                            )
                            self.pytorch_instruction.hide()
                            self.inference_demo_widgets.extend([self.pytorch_instruction])

                        # ONNX
                        local_onnx_demo = os.path.join(os.getcwd(), model_demo_path, ONNX_FILE_NAME)
                        if file_exists(local_onnx_demo):
                            if self.app_options.get("export_onnx_supported", False):
                                onnx_demo_link = f"{link_to_demo}/{ONNX_FILE_NAME}"
                                onnx_icon = Field.Icon(
                                    image_url=ONNX_ICON, bg_color_rgb=[255, 255, 255]
                                )
                                self.onnx_instruction = Field(
                                    title="ONNX",
                                    description="Open file",
                                    description_url=onnx_demo_link,
                                    icon=onnx_icon,
                                    content=Empty(),
                                )
                                self.onnx_instruction.hide()
                                self.inference_demo_widgets.extend([self.onnx_instruction])

                        # TensorRT
                        local_trt_demo = os.path.join(os.getcwd(), model_demo_path, TRT_FILE_NAME)
                        if file_exists(local_trt_demo):
                            if self.app_options.get("export_tensorrt_supported", False):
                                trt_demo_link = f"{link_to_demo}/{TRT_FILE_NAME}"
                                trt_icon = Field.Icon(
                                    image_url=TRT_ICON, bg_color_rgb=[255, 255, 255]
                                )
                                self.trt_instruction = Field(
                                    title="TensorRT",
                                    description="Open file",
                                    description_url=trt_demo_link,
                                    icon=trt_icon,
                                    content=Empty(),
                                )
                                self.trt_instruction.hide()
                                self.inference_demo_widgets.extend([self.trt_instruction])

                        local_demo_overview = os.path.join(
                            os.getcwd(), model_demo_path, OVERVIEW_FILE_NAME
                        )
                        if file_exists(local_demo_overview):
                            demo_overview_link = os.path.join(link_to_demo, OVERVIEW_FILE_NAME)
                        else:
                            demo_overview_link = None

                        self.inference_demo_field = Field(
                            title="How to run inference",
                            description="Instructions on how to use your checkpoints outside of Supervisely Platform",
                            content=Flexbox(self.inference_demo_widgets),
                            title_url=demo_overview_link,
                        )
                        self.inference_demo_field.hide()
                        self.display_widgets.extend([self.inference_demo_field])
        # -------------------------------- #

        self.container = Container(self.display_widgets)
        self.card = Card(
            title=self.title,
            description=self.description,
            content=self.container,
            lock_message=self.lock_message,
        )
        self.card.lock()

    @property
    def widgets_to_disable(self) -> list:
        return []

    def validate_step(self) -> bool:
        return True

    def overview_demo_exists(self, demo_path: str):
        return file_exists(os.path.join(os.getcwd(), demo_path, OVERVIEW_FILE_NAME))

    def pytorch_demo_exists(self, demo_path: str):
        return file_exists(os.path.join(os.getcwd(), demo_path, PYTORCH_FILE_NAME))

    def onnx_demo_exists(self, demo_path: str):
        return file_exists(os.path.join(os.getcwd(), demo_path, ONNX_FILE_NAME))

    def trt_demo_exists(self, demo_path: str):
        return file_exists(os.path.join(os.getcwd(), demo_path, TRT_FILE_NAME))



# coding: utf-8

# docs
from __future__ import annotations

import uuid
from typing import Dict, List, Optional, Tuple, Union

from supervisely._utils import take_with_default
from supervisely.annotation.tag import Tag, TagJsonFields
from supervisely.annotation.tag_meta import TagMeta, TagTargetType
from supervisely.annotation.tag_meta_collection import TagMetaCollection
from supervisely.api.module_api import ApiField
from supervisely.sly_logger import logger
from supervisely.video_annotation.constants import FRAME_RANGE, ID, KEY
from supervisely.video_annotation.key_id_map import KeyIdMap


class VideoTag(Tag):
    """
    VideoTag object for :class:`VideoAnnotation<supervisely.video_annotation.video_annotation.VideoAnnotation>`. :class:`VideoTag<VideoTag>` object is immutable.

    :param meta: General information about Video Tag.
    :type meta: TagMeta
    :param value: Video Tag value. Depends on :class:`TagValueType<TagValueType>` of :class:`TagMeta<TagMeta>`.
    :type value: str or int or float or None, optional
    :param frame_range: Video Tag frame range.
    :type frame_range: Tuple[int, int] or List[int, int], optional
    :param key: uuid.UUID object.
    :type key: uuid.UUID, optional
    :param sly_id: Video Tag ID in Supervisely.
    :type sly_id: int, optional
    :param labeler_login: Login of user who created VideoTag.
    :type labeler_login: str, optional
    :param updated_at: Date and Time when VideoTag was modified last. Date Format: Year:Month:Day:Hour:Minute:Seconds. Example: '2021-01-22T19:37:50.158Z'.
    :type updated_at: str, optional
    :param created_at: Date and Time when VideoTag was created. Date Format is the same as in "updated_at" parameter.
    :type created_at: str, optional
    :param is_finished: Video Tag is finished or not (applicable for range tags).
    :type is_finished: bool, optional
    :param non_final_value: Video Tag value is final or not. Can be useful to create tag without value.
    :type non_final_value: bool, optional
    :Usage example:

     .. code-block:: python

        import supervisely as sly
        from supervisely.video_annotation.video_tag import VideoTag

        meta_dog = sly.TagMeta('dog', sly.TagValueType.NONE)
        # Now we can create a VideoTag using our TagMeta
        tag_dog = VideoTag(meta_dog)
        # When you are creating a new Tag
        # Tag.value is automatically cross-checked against your TagMeta value type to make sure the value is valid.
        # If we now try to add a value to our newly created Tag, we receive "ValueError", because our TagMeta value type is "NONE"
        tag_dog = VideoTag(meta_dog, value="Husky")
        # Output: ValueError: Tag dog can not have value Husky

        # Let's create another Tag with a string value type and frame range
        meta_cat = sly.TagMeta('cat', sly.TagValueType.ANY_STRING)
        tag_cat = VideoTag(meta_cat, value="Fluffy", frame_range=(5, 10))

        # Now let's create a Tag using TagMeta with "ONEOF_STRING" value type
        # In order to use "oneof_string value type", you must initialize a variable with possible values(see class TagMeta for more information)
        colors = ["brown", "white", "black", "red", "chocolate", "gold", "grey"]
        meta_coat_color = sly.TagMeta('coat color', sly.TagValueType.ONEOF_STRING, possible_values=colors)
        tag_coat_color = VideoTag(meta_coat_color, value="white", frame_range=(15, 20))

        # If given value is not in a list of possible Tags, ValueError will be raised
        tag_coat_color = VideoTag(meta_coat_color, value="yellow")
        # Output: ValueError: Tag coat color can not have value yellow
    """

    _SUPPORT_UNFINISHED_TAGS = True

    def __init__(
        self,
        meta: TagMeta,
        value: Optional[Union[str, int, float]] = None,
        frame_range: Optional[Tuple[int, int]] = None,
        key: Optional[uuid.UUID] = None,
        sly_id: Optional[int] = None,
        labeler_login: Optional[str] = None,
        updated_at: Optional[str] = None,
        created_at: Optional[str] = None,
        is_finished: Optional[bool] = None,
        non_final_value: Optional[bool] = None,
    ):
        super(VideoTag, self).__init__(
            meta,
            value=value,
            sly_id=sly_id,
            labeler_login=labeler_login,
            updated_at=updated_at,
            created_at=created_at,
        )

        self._frame_range = None
        if frame_range is not None:
            self._frame_range = self._validate_frame_range(frame_range)

        self._key = take_with_default(key, uuid.uuid4())

        self._is_finished = None
        self._non_final_value = None

        if self._SUPPORT_UNFINISHED_TAGS is True:
            self._is_finished = take_with_default(is_finished, True)
            if self._is_finished is False:
                if self._frame_range is None:
                    raise ValueError("Cannot set is_finished=False for a tag without frame range")
                self._frame_range = [self._frame_range[0], self._frame_range[0]]
            if not self.meta.is_valid_value(self.value) and self.value is None:
                non_final_value = True
            self._non_final_value = take_with_default(non_final_value, False)
            if self._non_final_value is True:
                self._is_finished = False

    @property
    def frame_range(self) -> Tuple[int, int]:
        """
        VideoTag frame range.

        :return: Range of frames for current VideoTag
        :rtype: :class:`Tuple[int, int]`
        :Usage example:

         .. code-block:: python

            cat_range = tag_cat.frame_range # [5, 10]
        """
        return self._frame_range

    @property
    def is_finished(self) -> bool:
        """
        VideoTag is finished or not (applicable for range tags).

        :return: True if VideoTag is finished, otherwise False
        :rtype: :class:`bool`
        :Usage example:

         .. code-block:: python

            is_finished = tag_cat.is_finished
        """
        return self._is_finished

    @property
    def non_final_value(self) -> bool:
        """
        VideoTag value is final or not.

        :return: True if VideoTag value is final, otherwise False
        :rtype: :class:`bool`
        :Usage example:

         .. code-block:: python

            non_final_value = tag_cat.non_final_value
        """
        return self._non_final_value

    def key(self) -> uuid.UUID:
        return self._key

    def _validate_frame_range(self, frame_range: Tuple[int, int]) -> Tuple[int, int]:
        if not isinstance(frame_range, (tuple, list)):
            raise TypeError(
                f'frame_range has to be a tuple or a list. Given type "{type(frame_range)}".'
            )

        if (
            len(frame_range) != 2
            or not isinstance(frame_range[0], int)
            or not isinstance(frame_range[1], int)
        ):
            raise ValueError("frame_range has to be a tuple or a list with 2 int values.")
        return list(frame_range)

    def to_json(self, key_id_map: Optional[KeyIdMap] = None) -> Dict:
        """
        Convert the VideoTag to a json dict. Read more about `Supervisely format <https://docs.supervise.ly/data-organization/00_ann_format_navi>`_.

        :param key_id_map: Key ID Map object.
        :type key_id_map: KeyIdMap, optional
        :return: Json format as a dict
        :rtype: :class:`dict`
        :Usage example:

         .. code-block:: python

            import supervisely as sly
            from supervisely.video_annotation.video_tag import VideoTag
            meta_dog = sly.TagMeta('dog', sly.TagValueType.NONE)
            tag_dog = VideoTag(meta_dog)
            tag_dog_json = tag_dog.to_json()
            print(tag_dog_json)
            # Output: {
            #     "name": "dog",
            #     "key": "058ad7993a534082b4d94cc52542a97d"
            # }
        """
        data_json = super(VideoTag, self).to_json()
        if type(data_json) is str:
            # @TODO: case when tag has no value, super.to_json() returns tag name
            data_json = {TagJsonFields.TAG_NAME: data_json}
        if self.frame_range is not None:
            data_json[FRAME_RANGE] = self.frame_range

        if self.is_finished is not None:
            data_json[ApiField.IS_FINISHED] = self.is_finished
        if self.non_final_value is not None:
            data_json[ApiField.NON_FINAL_VALUE] = self.non_final_value
        data_json[KEY] = self.key().hex

        if key_id_map is not None:
            item_id = key_id_map.get_tag_id(self.key())
            if item_id is not None:
                data_json[ID] = item_id

        return data_json

    @classmethod
    def from_json(
        cls,
        data: Dict,
        tag_meta_collection: TagMetaCollection,
        key_id_map: Optional[KeyIdMap] = None,
    ) -> VideoTag:
        """
        Convert a json dict to VideoTag. Read more about `Supervisely format <https://docs.supervise.ly/data-organization/00_ann_format_navi>`_.

        :param data: VideoTag in json format as a dict.
        :type data: dict
        :param tag_meta_collection: :class:`TagMetaCollection<supervisely.annotation.tag_meta_collection.TagMetaCollection>` object.
        :type tag_meta_collection: TagMetaCollection
        :param key_id_map: Key ID Map object.
        :type key_id_map: KeyIdMap, optional
        :return: VideoTag object
        :rtype: :class:`VideoTag<VideoTag>`
        :Usage example:

         .. code-block:: python

            import supervisely as sly

            tag_cat_json = {
                "name": "cat",
                "value": "Fluffy",
                "frameRange": [
                    5,
                    10
                ]
            }

            from supervisely.video_annotation.video_tag import VideoTag
            meta_cat = sly.TagMeta('cat', sly.TagValueType.ANY_STRING)
            meta_collection = sly.TagMetaCollection([meta_cat])
            tag_cat = VideoTag.from_json(tag_cat_json, meta_collection)
        """
        temp = super(VideoTag, cls).from_json(data, tag_meta_collection)
        frame_range = data.get(FRAME_RANGE, None)
        is_finished = data.get(ApiField.IS_FINISHED, True)
        non_final_value = data.get(ApiField.NON_FINAL_VALUE, False)
        key = uuid.UUID(data[KEY]) if KEY in data else uuid.uuid4()

        if key_id_map is not None:
            key_id_map.add_tag(key, data.get(ID, None))

        return cls(
            meta=temp.meta,
            value=temp.value,
            frame_range=frame_range,
            key=key,
            sly_id=temp.sly_id,
            labeler_login=temp.labeler_login,
            updated_at=temp.updated_at,
            created_at=temp.created_at,
            is_finished=is_finished,
            non_final_value=non_final_value,
        )

    def get_compact_str(self) -> str:
        """
        Get string with information about VideoTag: name, value and range of frames.

        :return: Information about VideoTag object
        :rtype: :class:`str`
        :Usage example:

         .. code-block:: python

            import supervisely as sly
            from supervisely.video_annotation.video_tag import VideoTag
            meta_cat = sly.TagMeta('cat', sly.TagValueType.ANY_STRING)
            tag_cat = VideoTag(meta_cat, value="Fluffy", frame_range=(5, 10))
            compact_tag_cat = tag_cat.get_compact_str()
            print(compact_tag_cat) # cat:Fluffy[5 - 10]
        """
        res = super(VideoTag, self).get_compact_str()
        if self.frame_range is not None:
            res = "{}[{} - {}]".format(res, self.frame_range[0], self.frame_range[1])
        if self.is_finished is False:
            res += " (unfinished)"
        if self.non_final_value is True:
            res += " (non-final value)"
        return res

    def __eq__(self, other: VideoTag) -> bool:
        """
        Checks that 2 VideoTags are equal by comparing their meta, value and frame_range.

        :param other: VideoTag object.
        :type other: VideoTag
        :return: True if comparable objects are equal, otherwise False
        :rtype: :class:`bool`
        :Usage example:

         .. code-block:: python

            import supervisely as sly
            from supervisely.video_annotation.video_tag import VideoTag

            # Let's create 2 identical Tags
            meta_lemon_1 = sly.TagMeta('Lemon', sly.TagValueType.NONE)
            tag_lemon_1 = VideoTag(meta_lemon_1)

            meta_lemon_2 = sly.TagMeta('Lemon', sly.TagValueType.NONE)
            tag_lemon_2 = VideoTag(meta_lemon_2)

            # and 1 different Tag to compare them
            meta_cucumber = sly.TagMeta('Cucumber', sly.TagValueType.ANY_STRING)
            tag_cucumber = VideoTag(meta_cucumber, value="Fresh")

            # Compare identical Tags
            print(tag_lemon_1 == tag_lemon_2)      # True

            # Compare unidentical Tags
            print(tag_lemon_1 == tag_cucumber)     # False
        """
        unfinished_tag_comparison = True
        if self._SUPPORT_UNFINISHED_TAGS:
            unfinished_tag_comparison = (
                self.is_finished == other.is_finished
                and self.non_final_value == other.non_final_value
            )
        return (
            isinstance(other, VideoTag)
            and self.meta == other.meta
            and self.value == other.value
            and self.frame_range == other.frame_range
            and unfinished_tag_comparison
        )

    def clone(
        self,
        meta: Optional[TagMeta] = None,
        value: Optional[Union[str, int, float]] = None,
        frame_range: Optional[Tuple[int, int]] = None,
        key: Optional[uuid.UUID] = None,
        sly_id: Optional[int] = None,
        labeler_login: Optional[str] = None,
        updated_at: Optional[str] = None,
        created_at: Optional[str] = None,
        is_finished: Optional[bool] = None,
        non_final_value: Optional[bool] = None,
    ) -> VideoTag:
        """
        Makes a copy of VideoTag with new fields, if fields are given, otherwise it will use fields of the original VideoTag.

        :param meta: General information about VideoTag.
        :type meta: TagMeta, optional
        :param value: VideoTag value. Depends on :class:`TagValueType<TagValueType>` of :class:`TagMeta<TagMeta>`.
        :type value: str or int or float or None, optional
        :param frame_range: VideoTag frame range.
        :type frame_range: Tuple[int, int] or List[int, int], optional
        :param key: uuid.UUID object.
        :type key: uuid.UUID, optional
        :param sly_id: VideoTag ID in Supervisely.
        :type sly_id: int, optional
        :param labeler_login: Login of user who created VideoTag.
        :type labeler_login: str, optional
        :param updated_at: Date and Time when VideoTag was modified last. Date Format: Year:Month:Day:Hour:Minute:Seconds. Example: '2021-01-22T19:37:50.158Z'.
        :type updated_at: str, optional
        :param created_at: Date and Time when VideoTag was created. Date Format is the same as in "updated_at" parameter.
        :type created_at: str, optional
        :param is_finished: Video Tag is finished or not (applicable for range tags).
        :type is_finished: bool, optional
        :param non_final_value: Video Tag value is final or not (applicable for range tags).
        :type non_final_value: bool, optional
        :Usage example:

         .. code-block:: python

            import supervisely as sly
            from supervisely.video_annotation.video_tag import VideoTag

            meta_car = sly.TagMeta('car_tag', sly.TagValueType.ANY_STRING)
            car_tag = VideoTag(meta_car, value='acura', frame_range=(7, 9))

            meta_bus = sly.TagMeta('bus', sly.TagValueType.ANY_STRING)
            new_tag = car_tag.clone(meta=meta_bus, frame_range=(15, 129), key=car_tag.key())
            new_tag_json = new_tag.to_json()
            print(new_tag_json)
            # Output: {
            #     "name": "bus",
            #     "value": "acura",
            #     "frameRange": [
            #         15,
            #         129
            #     ],
            #     "key": "360438485fd34264921ca19bd43b0b71"
            # }
        """
        return self.__class__(
            meta=take_with_default(meta, self.meta),
            value=take_with_default(value, self.value),
            frame_range=take_with_default(frame_range, self.frame_range),
            key=take_with_default(key, self.key()),
            sly_id=take_with_default(sly_id, self.sly_id),
            labeler_login=take_with_default(labeler_login, self.labeler_login),
            updated_at=take_with_default(updated_at, self.updated_at),
            created_at=take_with_default(created_at, self.created_at),
            is_finished=take_with_default(is_finished, self.is_finished),
            non_final_value=take_with_default(non_final_value, self.non_final_value),
        )

    def __str__(self):
        # pylint: disable=too-many-format-args
        s = "{:<7s}{:<10}{:<7s} {:<13}{:<7s} {:<10} {:<12}".format(
            "Name:",
            self._meta.name,
            "Value type:",
            self._meta.value_type,
            "Value:",
            str(self.value),
            "FrameRange",
            str(self.frame_range),
        )
        if self._SUPPORT_UNFINISHED_TAGS:
            s += "{:<7s} {:<10} {:<12}".format(
                "Is finished:",
                str(self.is_finished),
                "Non final value:",
                str(self.non_final_value),
            )

        return s

    @classmethod
    def get_header_ptable(cls) -> List[str]:
        header = ["Name", "Value type", "Value", "Frame range"]
        if cls._SUPPORT_UNFINISHED_TAGS:
            header += ["Is finished", "Non final value"]
        return header

    def get_row_ptable(self) -> List[str]:
        row = [
            self._meta.name,
            self._meta.value_type,
            self.value,
            self.frame_range,
        ]
        if self._SUPPORT_UNFINISHED_TAGS:
            row += [self.is_finished, self.non_final_value]

        return row



# coding: utf-8

from typing import Optional

from supervisely.video_annotation.frame import Frame
from supervisely.video_annotation.key_id_map import KeyIdMap
from supervisely.volume_annotation.volume_figure import VolumeFigure
from supervisely.volume_annotation.volume_object_collection import VolumeObjectCollection
from supervisely.volume_annotation.constants import FIGURES, INDEX


class Slice(Frame):
    """
    A class representing a single slice of a medical image.
    Slice object for :class:`VolumeAnnotation<supervisely.volume_annotation.volume_annotation.VolumeAnnotation>`. :class:`Frame<Frame>` object is immutable.

    :param index: Index of the Slice.
    :type index: int
    :param figures: List of :class:`VolumeFigures<supervisely.volume_annotation.volume_figure.VolumeFigure>`.
    :type figures: Optional[List[VolumeFigure]]
    :Usage example:

     .. code-block:: python

        import supervisely as sly

        frame_index = 7
        geometry = sly.Rectangle(0, 0, 100, 100)
        class_car = sly.ObjClass('car', sly.Rectangle)
        object_car = sly.VolumeObject(class_car)
        figure_car = sly.VolumeFigure(object_car, geometry, frame_index)

        frame = sly.Slice(frame_index, figures=[figure_car])
        print(frame.to_json())
        # Output: {
        #     "figures": [
        #         {
        #         "geometry": {
        #             "points": {
        #             "exterior": [
        #                 [0, 0],
        #                 [100, 100]
        #             ],
        #             "interior": []
        #             }
        #         },
        #         "geometryType": "rectangle",
        #         "key": "eb0ab5f772054f70b6a9f5b583a47287",
        #         "meta": {
        #             "normal": { "x": 0, "y": 0, "z": 1 },
        #             "planeName": "axial",
        #             "sliceIndex": 7
        #         },
        #         "objectKey": "dbd236a6a6f440139fd0299905fcc46e"
        #         }
        #     ],
        #     "index": 7
        # }

    """

    figure_type = VolumeFigure

    # @classmethod
    # def from_json(cls, data, objects, slices_count=None, key_id_map=None):
    #     raise NotImplementedError()
    #     _frame = super().from_json(data, objects, slices_count, key_id_map)
    #     return cls(index=_frame.index, figures=_frame.figures)

    @classmethod
    def from_json(
        cls,
        data: dict,
        objects: VolumeObjectCollection,
        plane_name: str,
        slices_count: Optional[int] = None,
        key_id_map: Optional[KeyIdMap] = None,
    ):
        """
        Deserialize a `Slice` object from a JSON representation.

        :param data: The JSON representation of the `Slice`.
        :type data: dict
        :param objects: A collection of objects in volume.
        :type objects: VolumeObjectCollection
        :param plane_name: The name of the plane.
        :type plane_name: str
        :param slices_count: The total number of slices in the volume, if known.
        :type slices_count: Optional[int]
        :param key_id_map: A mapping of keys to IDs used for referencing objects.
        :type key_id_map: Optional[KeyIdMap]
        :return: The deserialized `Slice<Slice>` object.
        :rtype: Slice
        :raises ValueError: If the slice index is negative or greater than the total number of slices.

        :Usage example:

         .. code-block:: python

            import supervisely as sly
            slice_index = 7
            geometry = sly.Rectangle(0, 0, 100, 100)
            class_car = sly.ObjClass('car', sly.Rectangle)
            object_car = sly.VolumeObject(class_car)
            objects = sly.VolumeObjectCollection([object_car])
            figure_car = sly.VolumeFigure(object_car, geometry, sly.Plane.AXIAL, slice_index)

            slice = sly.Slice(slice_index, figures=[figure_car])
            slice_json = slice.to_json()

            new_slice = sly.Slice.from_json(slice_json, objects, sly.Plane.AXIAL)
        """

        index = data[INDEX]
        if index < 0:
            raise ValueError("Frame Index have to be >= 0")

        if slices_count is not None:
            if index > slices_count:
                raise ValueError(
                    "Item contains {} frames. Frame index is {}".format(slices_count, index)
                )

        figures = []
        for figure_json in data.get(FIGURES, []):
            figure = cls.figure_type.from_json(figure_json, objects, plane_name, index, key_id_map)
            figures.append(figure)
        return cls(index=index, figures=figures)



from typing import Any, Dict

import supervisely.io.env as sly_env
import supervisely.nn.training.gui.utils as gui_utils
from supervisely import Api, logger
from supervisely._utils import is_production
from supervisely.app.widgets import (
    Button,
    Card,
    Container,
    Progress,
    RunAppButton,
    TaskLogs,
    Text,
)


class TrainingLogs:
    title = "Training Logs"
    description = "Track training progress"
    lock_message = "Start training to unlock"

    def __init__(self, app_options: Dict[str, Any]):
        self.display_widgets = []
        api = Api.from_env()
        self.app_options = app_options

        # GUI Components
        self.validator_text = Text("")
        self.validator_text.hide()

        if is_production():
            task_id = sly_env.task_id(raise_not_found=False)
        else:
            task_id = None

        # Tensorboard button
        if is_production():
            task_info = api.task.get_info_by_id(task_id)
            session_token = task_info["meta"]["sessionToken"]
            sly_url_prefix = f"/net/{session_token}"
            self.tensorboard_link = f"{api.server_address}{sly_url_prefix}/tensorboard/"
        else:
            self.tensorboard_link = "http://localhost:8000/tensorboard"
        self.tensorboard_button = Button(
            "Open Tensorboard",
            button_type="info",
            plain=True,
            icon="zmdi zmdi-chart",
            link=self.tensorboard_link,
            visible_by_vue_field="!isStaticVersion",
        )
        self.tensorboard_button.disable()
        self.display_widgets.extend([self.validator_text, self.tensorboard_button])

        # Offline session Tensorboard button
        self.tensorboard_offline_button = None
        if is_production():
            workspace_id = sly_env.workspace_id()
            app_name = "Tensorboard Experiments Viewer"
            module_info = gui_utils.get_module_info_by_name(api, app_name)
            if module_info is not None:
                self.tensorboard_offline_button = RunAppButton(
                    workspace_id=workspace_id,
                    module_id=module_info["id"],
                    payload={},
                    text="Open Tensorboard",
                    button_type="info",
                    plain=True,
                    icon="zmdi zmdi-chart",
                    available_in_offline=True,
                    visible_by_vue_field="isStaticVersion",
                )
                self.tensorboard_offline_button.disable()
                self.display_widgets.extend([self.tensorboard_offline_button])
            else:
                logger.warning(
                    f"App '{app_name}' not found. Tensorboard button will not be displayed in offline mode."
                )

        # Optional Show logs button
        if app_options.get("show_logs_in_gui", False):
            self.logs_button = Button(
                text="Show logs",
                plain=True,
                button_size="mini",
                icon="zmdi zmdi-caret-down-circle",
            )
            self.task_logs = TaskLogs(task_id)
            self.task_logs.hide()
            logs_container = Container([self.logs_button, self.task_logs])
            self.display_widgets.extend([logs_container])
        # -------------------------------- #

        # Progress bars
        self.progress_bar_main = Progress(hide_on_finish=False)
        self.progress_bar_main.hide()
        self.progress_bar_secondary = Progress(hide_on_finish=False)
        self.progress_bar_secondary.hide()
        self.display_widgets.extend([self.progress_bar_main, self.progress_bar_secondary])
        # -------------------------------- #

        self.container = Container(self.display_widgets)
        self.card = Card(
            title=self.title,
            description=self.description,
            content=self.container,
            lock_message=self.lock_message,
        )
        self.card.lock()

    @property
    def widgets_to_disable(self) -> list:
        return []

    def validate_step(self) -> bool:
        return True

    def toggle_logs(self):
        if self.task_logs.is_hidden():
            self.task_logs.show()
            self.logs_button.text = "Hide logs"
            self.logs_button.icon = "zmdi zmdi-caret-up-circle"
        else:
            self.task_logs.hide()
            self.logs_button.text = "Show logs"
            self.logs_button.icon = "zmdi zmdi-caret-down-circle"



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: worker_api.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='worker_api.proto',
  package='',
  syntax='proto3',
  serialized_pb=_b('\n\x10worker_api.proto\"\x19\n\tModelName\x12\x0c\n\x04name\x18\x01 \x01(\t\"{\n\x11ModelArchitecture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0ctrain_docker\x18\x02 \x01(\t\x12\x12\n\ninf_docker\x18\x03 \x01(\t\x12\x15\n\rdeploy_docker\x18\x04 \x01(\t\x12\x17\n\x0f\x64\x65ploysm_docker\x18\x05 \x01(\t\"N\n\tModelInfo\x12\x1f\n\x04\x64\x65sc\x18\x01 \x01(\x0b\x32\x11.ModelDescription\x12 \n\x04\x61rch\x18\x02 \x01(\x0b\x32\x12.ModelArchitecture\",\n\x10ModelDescription\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04hash\x18\x02 \x01(\t\"\x19\n\tAgentInfo\x12\x0c\n\x04info\x18\x01 \x01(\t\"\x1a\n\nServerInfo\x12\x0c\n\x04info\x18\x01 \x01(\t\"N\n\x11ImagesInfoRequest\x12\x12\n\nproject_id\x18\x01 \x01(\r\x12\x12\n\ndataset_id\x18\x02 \x01(\r\x12\x11\n\timage_ids\x18\x03 \x03(\r\"#\n\nImagesInfo\x12\x15\n\x05infos\x18\x01 \x03(\x0b\x32\x06.Image\"\"\n\x04\x46ile\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04meta\x18\x02 \x01(\t\"!\n\tListFiles\x12\x14\n\x05\x66iles\x18\x01 \x03(\x0b\x32\x05.File\"6\n\rImportRequest\x12\x0f\n\x07task_id\x18\x01 \x01(\r\x12\x14\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x05.File\"7\n\tChunkFile\x12\x15\n\x05\x63hunk\x18\x01 \x01(\x0b\x32\x06.Chunk\x12\x13\n\x04\x66ile\x18\x02 \x01(\x0b\x32\x05.File\"E\n\nChunkModel\x12\x15\n\x05\x63hunk\x18\x01 \x01(\x0b\x32\x06.Chunk\x12 \n\x05model\x18\x02 \x01(\x0b\x32\x11.ModelDescription\"\x07\n\x05\x45mpty\"%\n\x06Result\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\r\n\x05\x65rror\x18\x02 \x01(\t\"\x10\n\x02Id\x12\n\n\x02id\x18\x01 \x01(\r\"s\n\x05Image\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04hash\x18\x02 \x01(\t\x12\r\n\x05title\x18\x03 \x01(\t\x12\x0b\n\x03\x65xt\x18\x04 \x01(\t\x12\x12\n\ndataset_id\x18\x05 \x01(\r\x12\x12\n\nproject_id\x18\x06 \x01(\r\x12\x0c\n\x04meta\x18\x07 \x01(\t\"\x1c\n\nImageArray\x12\x0e\n\x06images\x18\x01 \x03(\r\"M\n\x0bImagesToAdd\x12\x16\n\x06images\x18\x01 \x03(\x0b\x32\x06.Image\x12\x12\n\ndataset_id\x18\x02 \x01(\r\x12\x12\n\nproject_id\x18\x03 \x01(\r\"=\n\x0cImagesHashes\x12\x15\n\rimages_hashes\x18\x01 \x03(\t\x12\x16\n\x0esrc_node_token\x18\x02 \x01(\t\":\n\x05\x43hunk\x12\x0e\n\x06\x62uffer\x18\x01 \x01(\x0c\x12\x12\n\ntotal_size\x18\x02 \x01(\x04\x12\r\n\x05\x65rror\x18\x03 \x01(\t\":\n\nChunkImage\x12\x15\n\x05\x63hunk\x18\x01 \x01(\x0b\x32\x06.Chunk\x12\x15\n\x05image\x18\x02 \x01(\x0b\x32\x06.Image\"+\n\x0eNodeObjectHash\x12\x0c\n\x04hash\x18\x01 \x01(\t\x12\x0b\n\x03\x65xt\x18\x02 \x01(\t\"0\n\x0cGeneralEvent\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"2\n\x07Project\x12\n\n\x02id\x18\x01 \x01(\r\x12\r\n\x05title\x18\x02 \x01(\t\x12\x0c\n\x04meta\x18\x03 \x01(\t\"$\n\x07\x44\x61taset\x12\n\n\x02id\x18\x01 \x01(\r\x12\r\n\x05title\x18\x02 \x01(\t\"*\n\x0cProjectArray\x12\x1a\n\x08projects\x18\x01 \x03(\x0b\x32\x08.Project\"*\n\x0c\x44\x61tasetArray\x12\x1a\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x08.Dataset\"F\n\x0eProjectDataset\x12\x19\n\x07project\x18\x01 \x01(\x0b\x32\x08.Project\x12\x19\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x08.Dataset\"\"\n\x0cProjectStats\x12\x12\n\ncnt_images\x18\x01 \x01(\r\"\x18\n\x08LogLines\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\t\"\x14\n\x04Task\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t2\xc4\x0c\n\nGeneralAPI\x12 \n\rCreateProject\x12\x08.Project\x1a\x03.Id\"\x00\x12\'\n\rCreateDataset\x12\x0f.ProjectDataset\x1a\x03.Id\"\x00\x12(\n\x10GetProjectByName\x12\x08.Project\x1a\x08.Project\"\x00\x12/\n\x10GetDatasetByName\x12\x0f.ProjectDataset\x1a\x08.Dataset\"\x00\x12*\n\x0eGetModelByName\x12\n.ModelName\x1a\n.ModelInfo\"\x00\x12!\n\x0eGetProjectMeta\x12\x03.Id\x1a\x08.Project\"\x00\x12*\n\x12GetProjectDatasets\x12\x03.Id\x1a\r.DatasetArray\"\x00\x12&\n\x10GetDatasetImages\x12\x03.Id\x1a\x0b.ImageArray\"\x00\x12+\n\rGetImagesInfo\x12\x0b.ImageArray\x1a\x0b.ImagesInfo\"\x00\x12\x30\n\x0e\x44ownloadImages\x12\r.ImagesHashes\x1a\x0b.ChunkImage\"\x00\x30\x01\x12\x33\n\x13\x44ownloadAnnotations\x12\x0b.ImageArray\x1a\x0b.ChunkImage\"\x00\x30\x01\x12\x31\n\x0f\x46indImagesExist\x12\r.ImagesHashes\x1a\r.ImagesHashes\"\x00\x12\x39\n\x1a\x41\x64\x64\x45xistingImagesToDataset\x12\x0c.ImagesToAdd\x1a\x0b.ImageArray\"\x00\x12\x31\n\x11UploadAnnotations\x12\x0b.ChunkImage\x1a\x0b.ImageArray\"\x00(\x01\x12#\n\rUploadArchive\x12\x06.Chunk\x1a\x06.Empty\"\x00(\x01\x12\'\n\x0fGetProjectStats\x12\x03.Id\x1a\r.ProjectStats\"\x00\x12#\n\x12SetProjectFinished\x12\x03.Id\x1a\x06.Empty\"\x00\x12\x1a\n\x03Log\x12\t.LogLines\x1a\x06.Empty\"\x00\x12\'\n\x12GetImportStructure\x12\x03.Id\x1a\n.ListFiles\"\x00\x12\x30\n\x0eGetImportFiles\x12\x0e.ImportRequest\x1a\n.ChunkFile\"\x00\x30\x01\x12\x1f\n\nGetNewTask\x12\x06.Empty\x1a\x05.Task\"\x00\x30\x01\x12\x1e\n\x0bGetStopTask\x12\x06.Empty\x1a\x03.Id\"\x00\x30\x01\x12+\n\x0e\x41gentConnected\x12\n.AgentInfo\x1a\x0b.ServerInfo\"\x00\x12\x1d\n\tAgentPing\x12\x06.Empty\x1a\x06.Empty\"\x00\x12&\n\x0bUploadModel\x12\x0b.ChunkModel\x1a\x06.Empty\"\x00(\x01\x12.\n\rDownloadModel\x12\x11.ModelDescription\x1a\x06.Chunk\"\x00\x30\x01\x12\x31\n\x12GenerateNewModelId\x12\x06.Empty\x1a\x11.ModelDescription\"\x00\x12%\n\x10GetTelemetryTask\x12\x06.Empty\x1a\x05.Task\"\x00\x30\x01\x12\'\n\x0fUpdateTelemetry\x12\n.AgentInfo\x1a\x06.Empty\"\x00\x12\'\n\tAddImages\x12\x0b.ImagesInfo\x1a\x0b.ImageArray\"\x00\x12\'\n\x0cUploadImages\x12\x0b.ChunkImage\x1a\x06.Empty\"\x00(\x01\x12/\n\x10GetUsedImageList\x12\x06.Empty\x1a\x0f.NodeObjectHash\"\x00\x30\x01\x12/\n\x10GetUsedModelList\x12\x06.Empty\x1a\x0f.NodeObjectHash\"\x00\x30\x01\x12\x33\n\x16GetGeneralEventsStream\x12\x06.Empty\x1a\r.GeneralEvent\"\x00\x30\x01\x12)\n\x13GetGeneralEventData\x12\x06.Empty\x1a\x06.Chunk\"\x00\x30\x01\x12*\n\x14SendGeneralEventData\x12\x06.Chunk\x1a\x06.Empty\"\x00(\x01\x12&\n\x10\x41\x64\x64MetaToProject\x12\x08.Project\x1a\x06.Empty\"\x00\x62\x06proto3')
)




_MODELNAME = _descriptor.Descriptor(
  name='ModelName',
  full_name='ModelName',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ModelName.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=45,
)


_MODELARCHITECTURE = _descriptor.Descriptor(
  name='ModelArchitecture',
  full_name='ModelArchitecture',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='ModelArchitecture.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='train_docker', full_name='ModelArchitecture.train_docker', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inf_docker', full_name='ModelArchitecture.inf_docker', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploy_docker', full_name='ModelArchitecture.deploy_docker', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deploysm_docker', full_name='ModelArchitecture.deploysm_docker', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=47,
  serialized_end=170,
)


_MODELINFO = _descriptor.Descriptor(
  name='ModelInfo',
  full_name='ModelInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='desc', full_name='ModelInfo.desc', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='arch', full_name='ModelInfo.arch', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=172,
  serialized_end=250,
)


_MODELDESCRIPTION = _descriptor.Descriptor(
  name='ModelDescription',
  full_name='ModelDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ModelDescription.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='ModelDescription.hash', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=252,
  serialized_end=296,
)


_AGENTINFO = _descriptor.Descriptor(
  name='AgentInfo',
  full_name='AgentInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='info', full_name='AgentInfo.info', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=298,
  serialized_end=323,
)


_SERVERINFO = _descriptor.Descriptor(
  name='ServerInfo',
  full_name='ServerInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='info', full_name='ServerInfo.info', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=325,
  serialized_end=351,
)


_IMAGESINFOREQUEST = _descriptor.Descriptor(
  name='ImagesInfoRequest',
  full_name='ImagesInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_id', full_name='ImagesInfoRequest.project_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ImagesInfoRequest.dataset_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_ids', full_name='ImagesInfoRequest.image_ids', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=353,
  serialized_end=431,
)


_IMAGESINFO = _descriptor.Descriptor(
  name='ImagesInfo',
  full_name='ImagesInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='infos', full_name='ImagesInfo.infos', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=433,
  serialized_end=468,
)


_FILE = _descriptor.Descriptor(
  name='File',
  full_name='File',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='File.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meta', full_name='File.meta', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=504,
)


_LISTFILES = _descriptor.Descriptor(
  name='ListFiles',
  full_name='ListFiles',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='files', full_name='ListFiles.files', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=506,
  serialized_end=539,
)


_IMPORTREQUEST = _descriptor.Descriptor(
  name='ImportRequest',
  full_name='ImportRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task_id', full_name='ImportRequest.task_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='files', full_name='ImportRequest.files', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=595,
)


_CHUNKFILE = _descriptor.Descriptor(
  name='ChunkFile',
  full_name='ChunkFile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunk', full_name='ChunkFile.chunk', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='file', full_name='ChunkFile.file', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=597,
  serialized_end=652,
)


_CHUNKMODEL = _descriptor.Descriptor(
  name='ChunkModel',
  full_name='ChunkModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunk', full_name='ChunkModel.chunk', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model', full_name='ChunkModel.model', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=654,
  serialized_end=723,
)


_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=725,
  serialized_end=732,
)


_RESULT = _descriptor.Descriptor(
  name='Result',
  full_name='Result',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='Result.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='Result.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=734,
  serialized_end=771,
)


_ID = _descriptor.Descriptor(
  name='Id',
  full_name='Id',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Id.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=773,
  serialized_end=789,
)


_IMAGE = _descriptor.Descriptor(
  name='Image',
  full_name='Image',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Image.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hash', full_name='Image.hash', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='title', full_name='Image.title', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ext', full_name='Image.ext', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='Image.dataset_id', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='project_id', full_name='Image.project_id', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meta', full_name='Image.meta', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=791,
  serialized_end=906,
)


_IMAGEARRAY = _descriptor.Descriptor(
  name='ImageArray',
  full_name='ImageArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='images', full_name='ImageArray.images', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=908,
  serialized_end=936,
)


_IMAGESTOADD = _descriptor.Descriptor(
  name='ImagesToAdd',
  full_name='ImagesToAdd',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='images', full_name='ImagesToAdd.images', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataset_id', full_name='ImagesToAdd.dataset_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='project_id', full_name='ImagesToAdd.project_id', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=938,
  serialized_end=1015,
)


_IMAGESHASHES = _descriptor.Descriptor(
  name='ImagesHashes',
  full_name='ImagesHashes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='images_hashes', full_name='ImagesHashes.images_hashes', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='src_node_token', full_name='ImagesHashes.src_node_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1017,
  serialized_end=1078,
)


_CHUNK = _descriptor.Descriptor(
  name='Chunk',
  full_name='Chunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='buffer', full_name='Chunk.buffer', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_size', full_name='Chunk.total_size', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='Chunk.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1080,
  serialized_end=1138,
)


_CHUNKIMAGE = _descriptor.Descriptor(
  name='ChunkImage',
  full_name='ChunkImage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunk', full_name='ChunkImage.chunk', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image', full_name='ChunkImage.image', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1140,
  serialized_end=1198,
)


_NODEOBJECTHASH = _descriptor.Descriptor(
  name='NodeObjectHash',
  full_name='NodeObjectHash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='NodeObjectHash.hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ext', full_name='NodeObjectHash.ext', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1200,
  serialized_end=1243,
)


_GENERALEVENT = _descriptor.Descriptor(
  name='GeneralEvent',
  full_name='GeneralEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_id', full_name='GeneralEvent.request_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='GeneralEvent.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1245,
  serialized_end=1293,
)


_PROJECT = _descriptor.Descriptor(
  name='Project',
  full_name='Project',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Project.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='title', full_name='Project.title', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meta', full_name='Project.meta', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1295,
  serialized_end=1345,
)


_DATASET = _descriptor.Descriptor(
  name='Dataset',
  full_name='Dataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Dataset.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='title', full_name='Dataset.title', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1347,
  serialized_end=1383,
)


_PROJECTARRAY = _descriptor.Descriptor(
  name='ProjectArray',
  full_name='ProjectArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projects', full_name='ProjectArray.projects', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1385,
  serialized_end=1427,
)


_DATASETARRAY = _descriptor.Descriptor(
  name='DatasetArray',
  full_name='DatasetArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='DatasetArray.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1429,
  serialized_end=1471,
)


_PROJECTDATASET = _descriptor.Descriptor(
  name='ProjectDataset',
  full_name='ProjectDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='project', full_name='ProjectDataset.project', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataset', full_name='ProjectDataset.dataset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1473,
  serialized_end=1543,
)


_PROJECTSTATS = _descriptor.Descriptor(
  name='ProjectStats',
  full_name='ProjectStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cnt_images', full_name='ProjectStats.cnt_images', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1545,
  serialized_end=1579,
)


_LOGLINES = _descriptor.Descriptor(
  name='LogLines',
  full_name='LogLines',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='LogLines.data', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1581,
  serialized_end=1605,
)


_TASK = _descriptor.Descriptor(
  name='Task',
  full_name='Task',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Task.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1607,
  serialized_end=1627,
)

_MODELINFO.fields_by_name['desc'].message_type = _MODELDESCRIPTION
_MODELINFO.fields_by_name['arch'].message_type = _MODELARCHITECTURE
_IMAGESINFO.fields_by_name['infos'].message_type = _IMAGE
_LISTFILES.fields_by_name['files'].message_type = _FILE
_IMPORTREQUEST.fields_by_name['files'].message_type = _FILE
_CHUNKFILE.fields_by_name['chunk'].message_type = _CHUNK
_CHUNKFILE.fields_by_name['file'].message_type = _FILE
_CHUNKMODEL.fields_by_name['chunk'].message_type = _CHUNK
_CHUNKMODEL.fields_by_name['model'].message_type = _MODELDESCRIPTION
_IMAGESTOADD.fields_by_name['images'].message_type = _IMAGE
_CHUNKIMAGE.fields_by_name['chunk'].message_type = _CHUNK
_CHUNKIMAGE.fields_by_name['image'].message_type = _IMAGE
_PROJECTARRAY.fields_by_name['projects'].message_type = _PROJECT
_DATASETARRAY.fields_by_name['datasets'].message_type = _DATASET
_PROJECTDATASET.fields_by_name['project'].message_type = _PROJECT
_PROJECTDATASET.fields_by_name['dataset'].message_type = _DATASET
DESCRIPTOR.message_types_by_name['ModelName'] = _MODELNAME
DESCRIPTOR.message_types_by_name['ModelArchitecture'] = _MODELARCHITECTURE
DESCRIPTOR.message_types_by_name['ModelInfo'] = _MODELINFO
DESCRIPTOR.message_types_by_name['ModelDescription'] = _MODELDESCRIPTION
DESCRIPTOR.message_types_by_name['AgentInfo'] = _AGENTINFO
DESCRIPTOR.message_types_by_name['ServerInfo'] = _SERVERINFO
DESCRIPTOR.message_types_by_name['ImagesInfoRequest'] = _IMAGESINFOREQUEST
DESCRIPTOR.message_types_by_name['ImagesInfo'] = _IMAGESINFO
DESCRIPTOR.message_types_by_name['File'] = _FILE
DESCRIPTOR.message_types_by_name['ListFiles'] = _LISTFILES
DESCRIPTOR.message_types_by_name['ImportRequest'] = _IMPORTREQUEST
DESCRIPTOR.message_types_by_name['ChunkFile'] = _CHUNKFILE
DESCRIPTOR.message_types_by_name['ChunkModel'] = _CHUNKMODEL
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['Result'] = _RESULT
DESCRIPTOR.message_types_by_name['Id'] = _ID
DESCRIPTOR.message_types_by_name['Image'] = _IMAGE
DESCRIPTOR.message_types_by_name['ImageArray'] = _IMAGEARRAY
DESCRIPTOR.message_types_by_name['ImagesToAdd'] = _IMAGESTOADD
DESCRIPTOR.message_types_by_name['ImagesHashes'] = _IMAGESHASHES
DESCRIPTOR.message_types_by_name['Chunk'] = _CHUNK
DESCRIPTOR.message_types_by_name['ChunkImage'] = _CHUNKIMAGE
DESCRIPTOR.message_types_by_name['NodeObjectHash'] = _NODEOBJECTHASH
DESCRIPTOR.message_types_by_name['GeneralEvent'] = _GENERALEVENT
DESCRIPTOR.message_types_by_name['Project'] = _PROJECT
DESCRIPTOR.message_types_by_name['Dataset'] = _DATASET
DESCRIPTOR.message_types_by_name['ProjectArray'] = _PROJECTARRAY
DESCRIPTOR.message_types_by_name['DatasetArray'] = _DATASETARRAY
DESCRIPTOR.message_types_by_name['ProjectDataset'] = _PROJECTDATASET
DESCRIPTOR.message_types_by_name['ProjectStats'] = _PROJECTSTATS
DESCRIPTOR.message_types_by_name['LogLines'] = _LOGLINES
DESCRIPTOR.message_types_by_name['Task'] = _TASK
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ModelName = _reflection.GeneratedProtocolMessageType('ModelName', (_message.Message,), dict(
  DESCRIPTOR = _MODELNAME,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ModelName)
  ))
_sym_db.RegisterMessage(ModelName)

ModelArchitecture = _reflection.GeneratedProtocolMessageType('ModelArchitecture', (_message.Message,), dict(
  DESCRIPTOR = _MODELARCHITECTURE,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ModelArchitecture)
  ))
_sym_db.RegisterMessage(ModelArchitecture)

ModelInfo = _reflection.GeneratedProtocolMessageType('ModelInfo', (_message.Message,), dict(
  DESCRIPTOR = _MODELINFO,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ModelInfo)
  ))
_sym_db.RegisterMessage(ModelInfo)

ModelDescription = _reflection.GeneratedProtocolMessageType('ModelDescription', (_message.Message,), dict(
  DESCRIPTOR = _MODELDESCRIPTION,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ModelDescription)
  ))
_sym_db.RegisterMessage(ModelDescription)

AgentInfo = _reflection.GeneratedProtocolMessageType('AgentInfo', (_message.Message,), dict(
  DESCRIPTOR = _AGENTINFO,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:AgentInfo)
  ))
_sym_db.RegisterMessage(AgentInfo)

ServerInfo = _reflection.GeneratedProtocolMessageType('ServerInfo', (_message.Message,), dict(
  DESCRIPTOR = _SERVERINFO,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ServerInfo)
  ))
_sym_db.RegisterMessage(ServerInfo)

ImagesInfoRequest = _reflection.GeneratedProtocolMessageType('ImagesInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _IMAGESINFOREQUEST,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImagesInfoRequest)
  ))
_sym_db.RegisterMessage(ImagesInfoRequest)

ImagesInfo = _reflection.GeneratedProtocolMessageType('ImagesInfo', (_message.Message,), dict(
  DESCRIPTOR = _IMAGESINFO,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImagesInfo)
  ))
_sym_db.RegisterMessage(ImagesInfo)

File = _reflection.GeneratedProtocolMessageType('File', (_message.Message,), dict(
  DESCRIPTOR = _FILE,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:File)
  ))
_sym_db.RegisterMessage(File)

ListFiles = _reflection.GeneratedProtocolMessageType('ListFiles', (_message.Message,), dict(
  DESCRIPTOR = _LISTFILES,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ListFiles)
  ))
_sym_db.RegisterMessage(ListFiles)

ImportRequest = _reflection.GeneratedProtocolMessageType('ImportRequest', (_message.Message,), dict(
  DESCRIPTOR = _IMPORTREQUEST,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImportRequest)
  ))
_sym_db.RegisterMessage(ImportRequest)

ChunkFile = _reflection.GeneratedProtocolMessageType('ChunkFile', (_message.Message,), dict(
  DESCRIPTOR = _CHUNKFILE,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ChunkFile)
  ))
_sym_db.RegisterMessage(ChunkFile)

ChunkModel = _reflection.GeneratedProtocolMessageType('ChunkModel', (_message.Message,), dict(
  DESCRIPTOR = _CHUNKMODEL,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ChunkModel)
  ))
_sym_db.RegisterMessage(ChunkModel)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), dict(
  DESCRIPTOR = _EMPTY,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Empty)
  ))
_sym_db.RegisterMessage(Empty)

Result = _reflection.GeneratedProtocolMessageType('Result', (_message.Message,), dict(
  DESCRIPTOR = _RESULT,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Result)
  ))
_sym_db.RegisterMessage(Result)

Id = _reflection.GeneratedProtocolMessageType('Id', (_message.Message,), dict(
  DESCRIPTOR = _ID,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Id)
  ))
_sym_db.RegisterMessage(Id)

Image = _reflection.GeneratedProtocolMessageType('Image', (_message.Message,), dict(
  DESCRIPTOR = _IMAGE,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Image)
  ))
_sym_db.RegisterMessage(Image)

ImageArray = _reflection.GeneratedProtocolMessageType('ImageArray', (_message.Message,), dict(
  DESCRIPTOR = _IMAGEARRAY,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImageArray)
  ))
_sym_db.RegisterMessage(ImageArray)

ImagesToAdd = _reflection.GeneratedProtocolMessageType('ImagesToAdd', (_message.Message,), dict(
  DESCRIPTOR = _IMAGESTOADD,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImagesToAdd)
  ))
_sym_db.RegisterMessage(ImagesToAdd)

ImagesHashes = _reflection.GeneratedProtocolMessageType('ImagesHashes', (_message.Message,), dict(
  DESCRIPTOR = _IMAGESHASHES,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ImagesHashes)
  ))
_sym_db.RegisterMessage(ImagesHashes)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), dict(
  DESCRIPTOR = _CHUNK,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Chunk)
  ))
_sym_db.RegisterMessage(Chunk)

ChunkImage = _reflection.GeneratedProtocolMessageType('ChunkImage', (_message.Message,), dict(
  DESCRIPTOR = _CHUNKIMAGE,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ChunkImage)
  ))
_sym_db.RegisterMessage(ChunkImage)

NodeObjectHash = _reflection.GeneratedProtocolMessageType('NodeObjectHash', (_message.Message,), dict(
  DESCRIPTOR = _NODEOBJECTHASH,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:NodeObjectHash)
  ))
_sym_db.RegisterMessage(NodeObjectHash)

GeneralEvent = _reflection.GeneratedProtocolMessageType('GeneralEvent', (_message.Message,), dict(
  DESCRIPTOR = _GENERALEVENT,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:GeneralEvent)
  ))
_sym_db.RegisterMessage(GeneralEvent)

Project = _reflection.GeneratedProtocolMessageType('Project', (_message.Message,), dict(
  DESCRIPTOR = _PROJECT,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Project)
  ))
_sym_db.RegisterMessage(Project)

Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), dict(
  DESCRIPTOR = _DATASET,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Dataset)
  ))
_sym_db.RegisterMessage(Dataset)

ProjectArray = _reflection.GeneratedProtocolMessageType('ProjectArray', (_message.Message,), dict(
  DESCRIPTOR = _PROJECTARRAY,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ProjectArray)
  ))
_sym_db.RegisterMessage(ProjectArray)

DatasetArray = _reflection.GeneratedProtocolMessageType('DatasetArray', (_message.Message,), dict(
  DESCRIPTOR = _DATASETARRAY,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:DatasetArray)
  ))
_sym_db.RegisterMessage(DatasetArray)

ProjectDataset = _reflection.GeneratedProtocolMessageType('ProjectDataset', (_message.Message,), dict(
  DESCRIPTOR = _PROJECTDATASET,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ProjectDataset)
  ))
_sym_db.RegisterMessage(ProjectDataset)

ProjectStats = _reflection.GeneratedProtocolMessageType('ProjectStats', (_message.Message,), dict(
  DESCRIPTOR = _PROJECTSTATS,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:ProjectStats)
  ))
_sym_db.RegisterMessage(ProjectStats)

LogLines = _reflection.GeneratedProtocolMessageType('LogLines', (_message.Message,), dict(
  DESCRIPTOR = _LOGLINES,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:LogLines)
  ))
_sym_db.RegisterMessage(LogLines)

Task = _reflection.GeneratedProtocolMessageType('Task', (_message.Message,), dict(
  DESCRIPTOR = _TASK,
  __module__ = 'worker_api_pb2'
  # @@protoc_insertion_point(class_scope:Task)
  ))
_sym_db.RegisterMessage(Task)



_GENERALAPI = _descriptor.ServiceDescriptor(
  name='GeneralAPI',
  full_name='GeneralAPI',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=1630,
  serialized_end=3234,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateProject',
    full_name='GeneralAPI.CreateProject',
    index=0,
    containing_service=None,
    input_type=_PROJECT,
    output_type=_ID,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateDataset',
    full_name='GeneralAPI.CreateDataset',
    index=1,
    containing_service=None,
    input_type=_PROJECTDATASET,
    output_type=_ID,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetProjectByName',
    full_name='GeneralAPI.GetProjectByName',
    index=2,
    containing_service=None,
    input_type=_PROJECT,
    output_type=_PROJECT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatasetByName',
    full_name='GeneralAPI.GetDatasetByName',
    index=3,
    containing_service=None,
    input_type=_PROJECTDATASET,
    output_type=_DATASET,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetModelByName',
    full_name='GeneralAPI.GetModelByName',
    index=4,
    containing_service=None,
    input_type=_MODELNAME,
    output_type=_MODELINFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetProjectMeta',
    full_name='GeneralAPI.GetProjectMeta',
    index=5,
    containing_service=None,
    input_type=_ID,
    output_type=_PROJECT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetProjectDatasets',
    full_name='GeneralAPI.GetProjectDatasets',
    index=6,
    containing_service=None,
    input_type=_ID,
    output_type=_DATASETARRAY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatasetImages',
    full_name='GeneralAPI.GetDatasetImages',
    index=7,
    containing_service=None,
    input_type=_ID,
    output_type=_IMAGEARRAY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetImagesInfo',
    full_name='GeneralAPI.GetImagesInfo',
    index=8,
    containing_service=None,
    input_type=_IMAGEARRAY,
    output_type=_IMAGESINFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DownloadImages',
    full_name='GeneralAPI.DownloadImages',
    index=9,
    containing_service=None,
    input_type=_IMAGESHASHES,
    output_type=_CHUNKIMAGE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DownloadAnnotations',
    full_name='GeneralAPI.DownloadAnnotations',
    index=10,
    containing_service=None,
    input_type=_IMAGEARRAY,
    output_type=_CHUNKIMAGE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='FindImagesExist',
    full_name='GeneralAPI.FindImagesExist',
    index=11,
    containing_service=None,
    input_type=_IMAGESHASHES,
    output_type=_IMAGESHASHES,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddExistingImagesToDataset',
    full_name='GeneralAPI.AddExistingImagesToDataset',
    index=12,
    containing_service=None,
    input_type=_IMAGESTOADD,
    output_type=_IMAGEARRAY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadAnnotations',
    full_name='GeneralAPI.UploadAnnotations',
    index=13,
    containing_service=None,
    input_type=_CHUNKIMAGE,
    output_type=_IMAGEARRAY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadArchive',
    full_name='GeneralAPI.UploadArchive',
    index=14,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetProjectStats',
    full_name='GeneralAPI.GetProjectStats',
    index=15,
    containing_service=None,
    input_type=_ID,
    output_type=_PROJECTSTATS,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SetProjectFinished',
    full_name='GeneralAPI.SetProjectFinished',
    index=16,
    containing_service=None,
    input_type=_ID,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Log',
    full_name='GeneralAPI.Log',
    index=17,
    containing_service=None,
    input_type=_LOGLINES,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetImportStructure',
    full_name='GeneralAPI.GetImportStructure',
    index=18,
    containing_service=None,
    input_type=_ID,
    output_type=_LISTFILES,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetImportFiles',
    full_name='GeneralAPI.GetImportFiles',
    index=19,
    containing_service=None,
    input_type=_IMPORTREQUEST,
    output_type=_CHUNKFILE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetNewTask',
    full_name='GeneralAPI.GetNewTask',
    index=20,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_TASK,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetStopTask',
    full_name='GeneralAPI.GetStopTask',
    index=21,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_ID,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AgentConnected',
    full_name='GeneralAPI.AgentConnected',
    index=22,
    containing_service=None,
    input_type=_AGENTINFO,
    output_type=_SERVERINFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AgentPing',
    full_name='GeneralAPI.AgentPing',
    index=23,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadModel',
    full_name='GeneralAPI.UploadModel',
    index=24,
    containing_service=None,
    input_type=_CHUNKMODEL,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DownloadModel',
    full_name='GeneralAPI.DownloadModel',
    index=25,
    containing_service=None,
    input_type=_MODELDESCRIPTION,
    output_type=_CHUNK,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GenerateNewModelId',
    full_name='GeneralAPI.GenerateNewModelId',
    index=26,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_MODELDESCRIPTION,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetTelemetryTask',
    full_name='GeneralAPI.GetTelemetryTask',
    index=27,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_TASK,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateTelemetry',
    full_name='GeneralAPI.UpdateTelemetry',
    index=28,
    containing_service=None,
    input_type=_AGENTINFO,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddImages',
    full_name='GeneralAPI.AddImages',
    index=29,
    containing_service=None,
    input_type=_IMAGESINFO,
    output_type=_IMAGEARRAY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadImages',
    full_name='GeneralAPI.UploadImages',
    index=30,
    containing_service=None,
    input_type=_CHUNKIMAGE,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetUsedImageList',
    full_name='GeneralAPI.GetUsedImageList',
    index=31,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_NODEOBJECTHASH,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetUsedModelList',
    full_name='GeneralAPI.GetUsedModelList',
    index=32,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_NODEOBJECTHASH,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetGeneralEventsStream',
    full_name='GeneralAPI.GetGeneralEventsStream',
    index=33,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GENERALEVENT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetGeneralEventData',
    full_name='GeneralAPI.GetGeneralEventData',
    index=34,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_CHUNK,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SendGeneralEventData',
    full_name='GeneralAPI.SendGeneralEventData',
    index=35,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddMetaToProject',
    full_name='GeneralAPI.AddMetaToProject',
    index=36,
    containing_service=None,
    input_type=_PROJECT,
    output_type=_EMPTY,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_GENERALAPI)

DESCRIPTOR.services_by_name['GeneralAPI'] = _GENERALAPI

# @@protoc_insertion_point(module_scope)



import torch  # pylint: disable=import-error

try:
    import fastreid
except ImportError:
    import sys
    from pathlib import Path

    fast_reid_repo_url = "https://github.com/supervisely-ecosystem/fast-reid.git"
    fast_reid_parent_path = Path(__file__).parent
    fast_reid_path = fast_reid_parent_path.joinpath("fast_reid")
    if not fast_reid_path.exists():
        import subprocess

        subprocess.run(["git", "clone", fast_reid_repo_url, str(fast_reid_path.resolve())])

    sys.path.insert(0, str(fast_reid_path.resolve()))

    import fastreid

from supervisely.nn.tracker.bot_sort.sly_tracker import BoTTracker



import supervisely.nn.artifacts as artifacts
import supervisely.nn.benchmark as benchmark
import supervisely.nn.inference as inference
from supervisely.nn.artifacts.artifacts import BaseTrainArtifacts, TrainInfo
from supervisely.nn.prediction_dto import (
    Prediction,
    PredictionAlphaMask,
    PredictionBBox,
    PredictionCuboid3d,
    PredictionKeypoints,
    PredictionMask,
    PredictionSegmentation,
    ProbabilityMask,
)
from supervisely.nn.task_type import TaskType
from supervisely.nn.utils import ModelSource, RuntimeType
from supervisely.nn.experiments import ExperimentInfo, get_experiment_infos



import imghdr
import os
from typing import Dict, List, Optional, Set, Tuple, Union

from supervisely import (
    Api,
    PointcloudEpisodeAnnotation,
    batched,
    generate_free_name,
    is_development,
    logger,
)
from supervisely.api.module_api import ApiField
from supervisely.convert.base_converter import BaseConverter
from supervisely.io.fs import get_file_ext, get_file_name
from supervisely.io.json import load_json_file
from supervisely.pointcloud.pointcloud import ALLOWED_POINTCLOUD_EXTENSIONS
from supervisely.pointcloud.pointcloud import validate_ext as validate_pcd_ext
from supervisely.project.project_settings import LabelingInterface


class PointcloudEpisodeConverter(BaseConverter):
    allowed_exts = ALLOWED_POINTCLOUD_EXTENSIONS
    modality = "pointcloud episodes"

    class Item(BaseConverter.BaseItem):
        def __init__(
            self,
            item_path,
            frame_number: int,
            ann_data: Optional[str] = None,
            related_images: Optional[list] = None,
            custom_data: Optional[dict] = None,
        ):
            self._name: str = None
            self._path = item_path
            self._frame_number = frame_number
            self._ann_data = ann_data
            self._related_images = related_images if related_images is not None else []
            self._type = "point_cloud_episode"
            self._custom_data = custom_data if custom_data is not None else {}

        @property
        def frame_number(self) -> int:
            return self._frame_number

        def create_empty_annotation(self) -> PointcloudEpisodeAnnotation:
            return PointcloudEpisodeAnnotation()

        def set_related_images(self, related_images: dict) -> None:
            self._related_images.append(related_images)

    def __init__(
        self,
        input_data: str,
        labeling_interface: Optional[Union[LabelingInterface, str]],
        upload_as_links: bool = False,
        remote_files_map: Optional[Dict[str, str]] = None,
    ):
        super().__init__(input_data, labeling_interface, upload_as_links, remote_files_map)
        self._annotation = None
        self._frame_pointcloud_map = None
        self._frame_count = None

    @property
    def format(self):
        return self._converter.format

    @property
    def frame_count(self):
        if self._frame_count is None:
            self._frame_count = len(self._items)
        return self._frame_count

    @property
    def ann_ext(self):
        return None

    @property
    def key_file_ext(self):
        return None

    @staticmethod
    def validate_ann_file(ann_path, meta=None):
        return False

    def upload_dataset(
        self,
        api: Api,
        dataset_id: int,
        batch_size: int = 1,
        log_progress=True,
    ):
        """Upload converted data to Supervisely"""

        meta, renamed_classes, renamed_tags = self.merge_metas_with_conflicts(api, dataset_id)

        existing_names = set([pcde.name for pcde in api.pointcloud_episode.get_list(dataset_id)])

        if log_progress:
            progress, progress_cb = self.get_progress(
                self.items_count, "Uploading pointcloud episodes..."
            )
        else:
            progress_cb = None

        frame_to_pointcloud_ids = {}
        for batch in batched(self._items, batch_size=batch_size):
            item_names = []
            item_paths = []
            item_metas = []
            for item in batch:
                item.name = generate_free_name(
                    existing_names, item.name, with_ext=True, extend_used_names=True
                )
                item_names.append(item.name)
                item_paths.append(item.path)

            pcd_infos = api.pointcloud_episode.upload_paths(
                dataset_id,
                item_names,
                item_paths,
            )
            pcd_ids = [pcd_info.id for pcd_info in pcd_infos]

            for item, pcd_id in zip(batch, pcd_ids):
                frame_to_pointcloud_ids[item._frame_number] = pcd_id
                item_metas.append({"frame": item._frame_number})
                rimg_infos = []
                camera_names = []
                if len(item._related_images) > 0:
                    img_paths, rimg_ann_paths = list(zip(*item._related_images))
                    rimg_hashes = api.pointcloud_episode.upload_related_images(img_paths)
                    for img_ind, (img_hash, rimg_ann_path) in enumerate(
                        zip(rimg_hashes, rimg_ann_paths)
                    ):
                        meta_json = load_json_file(rimg_ann_path)
                        try:
                            if ApiField.META not in meta_json:
                                raise ValueError("Related image meta not found in json file.")
                            if ApiField.NAME not in meta_json:
                                raise ValueError("Related image name not found in json file.")
                            if "deviceId" not in meta_json[ApiField.META].keys():
                                camera_names.append(f"CAM_{str(img_ind).zfill(2)}")
                            else:
                                camera_names.append(meta_json[ApiField.META]["deviceId"])
                            rimg_infos.append(
                                {
                                    ApiField.ENTITY_ID: pcd_id,
                                    ApiField.NAME: meta_json[ApiField.NAME],
                                    ApiField.HASH: img_hash,
                                    ApiField.META: meta_json[ApiField.META],
                                }
                            )
                            api.pointcloud.add_related_images(rimg_infos, camera_names)
                        except Exception as e:
                            logger.warning(
                                f"Failed to upload related image or add it to pointcloud episo: {repr(e)}"
                            )
                            continue

            if log_progress:
                progress_cb(len(batch))

        if self.items_count > 0:
            ann = self.to_supervisely(self._items[0], meta, renamed_classes, renamed_tags)
            api.pointcloud_episode.annotation.append(dataset_id, ann, frame_to_pointcloud_ids)

        if log_progress:
            if is_development():
                progress.close()
        logger.info(f"Dataset ID:{dataset_id} has been successfully uploaded.")

    def _collect_items_if_format_not_detected(self) -> Tuple[List[Item], bool, Set[str]]:
        only_modality_items = True
        unsupported_exts = set()
        pcd_dict = {}
        frames_pcd_map = None
        used_img_ext = set()
        rimg_dict, rimg_json_dict = {}, {}
        for root, _, files in os.walk(self._input_data):
            for file in files:
                full_path = os.path.join(root, file)
                if file == "frame_pointcloud_map.json":
                    frames_pcd_map = load_json_file(full_path)
                    continue

                ext = get_file_ext(full_path)
                recognized_ext = imghdr.what(full_path)
                if ext == ".json":
                    rimg_json_dict[file] = full_path
                elif recognized_ext:
                    if ext.lower() == ".pcd":
                        logger.warning(
                            f"File '{file}' has been recognized as '.{recognized_ext}' format. Skipping."
                        )
                        continue
                    rimg_dict[file] = full_path
                    if ext not in used_img_ext:
                        used_img_ext.add(ext)
                elif ext.lower() in self.allowed_exts:
                    try:
                        validate_pcd_ext(ext)
                        pcd_dict[file] = full_path
                    except:
                        pass
                else:
                    only_modality_items = False
                    unsupported_exts.add(ext)

        items = []
        updated_frames_pcd_map = {}
        if frames_pcd_map:
            list_of_pcd_names = list(frames_pcd_map.values())
        else:
            list_of_pcd_names = sorted(pcd_dict.keys())

        for i, pcd_name in enumerate(list_of_pcd_names):
            if pcd_name in pcd_dict:
                updated_frames_pcd_map[i] = pcd_name
                item = self.Item(pcd_dict[pcd_name], i)
                for ext in used_img_ext:
                    rimg_name = f"{item.name}{ext}"
                    if not rimg_name in rimg_dict:
                        rimg_name = f"{get_file_name(item.name)}{ext}"
                    if rimg_name in rimg_dict:
                        rimg_path = rimg_dict[rimg_name]
                        rimg_ann_name = f"{rimg_name}.json"
                        if rimg_ann_name in rimg_json_dict:
                            rimg_ann_path = rimg_json_dict[rimg_ann_name]
                            item.set_related_images((rimg_path, rimg_ann_path))
                items.append(item)
            else:
                logger.warning(f"Pointcloud file {pcd_name} not found. Skipping frame.")
                continue
        self._frame_pointcloud_map = updated_frames_pcd_map
        self._frame_count = len(self._frame_pointcloud_map)

        return items, only_modality_items, unsupported_exts
